<?php
/**
 * This file is part of the official Amazon Pay and Login with Amazon extension
 * for Magento 1.x
 *
 * (c) 2019 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonPayments
 * @copyright  2019 creativestyle GmbH
 * @author     Marek Zabrowarny <ticket@creativestyle.de>
 */

class Creativestyle_AmazonPayments_Model_Url extends Mage_Core_Model_Abstract
{
    const LOGIN_CALLBACK_PATH = 'amazonpayments/login';
    const LOGIN_REDIRECT_PATH = 'amazonpayments/advanced_login/redirect';

    const ACCOUNT_PATH = 'customer/account';
    const ACCOUNT_LOGIN_PATH = 'customer/account/login';

    const CART_PATH = 'checkout/cart';

    const CHECKOUT_PATH = 'amazonpayments/checkout';
    const CHECKOUT_SAVE_SHIPPING_PATH = 'amazonpayments/checkout/saveShipping';
    const CHECKOUT_SAVE_SHIPPING_METHOD_PATH = 'amazonpayments/checkout/saveShippingMethod';
    const CHECKOUT_SAVE_ORDER_PATH = 'amazonpayments/checkout/saveOrder';
    const CHECKOUT_SAVE_COUPON_PATH = 'amazonpayments/checkout/couponPost';
    const CHECKOUT_INVALID_PAYMENT_PATH = 'amazonpayments/checkout/invalidPayment';
    const CHECKOUT_CANCEL_ORDER_REFERENCE_PATH = 'amazonpayments/checkout/cancelOrderReference';
    const CHECKOUT_MULTI_FACTOR_AUTHENTICATION_PATH = 'amazonpayments/checkout/mfa';

    const ACCESS_TOKEN_PARAM = 'accessToken';
    const ORDER_REFERENCE_ID_PARAM = 'orderReferenceId';

    /**
     * @var Mage_Core_Model_Url|null
     */
    protected $_urlModel = null;

    /**
     * Returns URL for the requested path and params generated by Magento URL model
     *
     * @param string|null $routePath
     * @param array|null $routeParams
     * @return string
     */
    protected function _getUrl($routePath, $routeParams = null)
    {
        if (null === $this->_urlModel) {
            $this->_urlModel = Mage::getModel('core/url');
        }

        return $this->_urlModel->getUrl($routePath, $routeParams);
    }

    /**
     * @return string
     */
    public function getLoginCallbackUrl()
    {
        return $this->_getUrl(self::LOGIN_CALLBACK_PATH);
    }

    /**
     * @return string
     */
    public function getLoginRedirectUrl()
    {
        return $this->_getUrl(self::LOGIN_REDIRECT_PATH);
    }

    /**
     * @return string
     */
    public function getLoginSuccessUrl()
    {
        return $this->_getUrl(self::ACCOUNT_PATH);
    }

    /**
     * @return string
     */
    public function getLoginFailureUrl()
    {
        return $this->_getUrl(self::ACCOUNT_LOGIN_PATH);
    }

    /**
     * @return string
     */
    public function getPayCallbackUrl()
    {
        return $this->getCheckoutUrl(array(self::ORDER_REFERENCE_ID_PARAM => '%s'));
    }

    /**
     * @param string $accessToken
     * @return string
     */
    public function getPaySuccessUrl($accessToken)
    {
        return $this->getCheckoutUrl(array(self::ACCESS_TOKEN_PARAM => $accessToken));
    }

    /**
     * @return string
     */
    public function getPayFailureUrl()
    {
        return $this->getCartUrl();
    }

    /**
     * @return string
     */
    public function getCartUrl()
    {
        return $this->_getUrl(self::CART_PATH);
    }

    /**
     * @param array|null $params
     * @return string
     */
    public function getCheckoutUrl($params = null)
    {
        return $this->_getUrl(self::CHECKOUT_PATH, $params);
    }

    /**
     * @return string
     */
    public function getCheckoutFailureUrl()
    {
        return $this->getCartUrl();
    }

    /**
     * @return string
     */
    public function getCheckoutSaveShippingUrl()
    {
        return $this->_getUrl(self::CHECKOUT_SAVE_SHIPPING_PATH);
    }

    /**
     * @return string
     */
    public function getCheckoutSaveShippingMethodUrl()
    {
        return $this->_getUrl(self::CHECKOUT_SAVE_SHIPPING_METHOD_PATH);
    }

    /**
     * @return string
     */
    public function getCheckoutSaveOrderUrl()
    {
        return $this->_getUrl(self::CHECKOUT_SAVE_ORDER_PATH);
    }

    /**
     * @return string
     */
    public function getCheckoutSaveCouponUrl()
    {
        return $this->_getUrl(self::CHECKOUT_SAVE_COUPON_PATH);
    }

    /**
     * @return string
     */
    public function getCheckoutInvalidPaymentUrl()
    {
        return $this->_getUrl(self::CHECKOUT_INVALID_PAYMENT_PATH);
    }

    /**
     * @return string
     */
    public function getCheckoutCancelOrderReferenceUrl()
    {
        return $this->_getUrl(self::CHECKOUT_CANCEL_ORDER_REFERENCE_PATH);
    }

    /**
     * @return string
     */
    public function getCheckoutMultiFactorAuthenticationUrl()
    {
        return $this->_getUrl(self::CHECKOUT_MULTI_FACTOR_AUTHENTICATION_PATH);
    }

}
